@@ -18,12 +18,13 @@ License for the specific language governing permissions and limitations
 under the License.
 '''
 
-import os
 import stat
-from .utils import fread, fwrite, RTSLibError, RTSLibNotInCFS
+from pathlib import Path
+
+from .utils import RTSLibError, RTSLibNotInCFSError, fread, fwrite
 
 
-class CFSNode(object):
+class CFSNode:
 
     # Where is the configfs base LIO directory ?
     configfs_dir = '/sys/kernel/config/target'
@@ -50,83 +51,106 @@ class CFSNode(object):
         lookup -> make sure it does NOT exist
         create -> create the node which must not exist beforehand
         '''
-        if mode not in ['any', 'lookup', 'create']:
-            raise RTSLibError("Invalid mode: %s" % mode)
+        if mode not in ('any', 'lookup', 'create'):
+            raise RTSLibError(f"Invalid mode: {mode}")
+
         if self.exists and mode == 'create':
-            raise RTSLibError("This %s already exists in configFS"
-                              % self.__class__.__name__)
+            # ensure that self.path is not stale hba-only dir
+            path = Path(self._path)
+            if path.resolve().parent.samefile(Path(self.configfs_dir) / 'core') \
+                    and not any(path.iterdir()):
+                path.rmdir()
+            else:
+                raise RTSLibError(f"This {self.__class__.__name__} already exists in configFS")
+
         elif not self.exists and mode == 'lookup':
-            raise RTSLibNotInCFS("No such %s in configfs: %s"
-                                 % (self.__class__.__name__, self.path))
+            raise RTSLibNotInCFSError(
+                f"No such {self.__class__.__name__} in configfs: {self.path}")
 
         if not self.exists:
             try:
-                os.mkdir(self.path)
-            except:
-                raise RTSLibError("Could not create %s in configFS"
-                                  % self.__class__.__name__)
+                Path(self.path).mkdir()
+            except Exception as e:
+                raise RTSLibError(f"Could not create {self.__class__.__name__} in configFS: {e}")
 
     def _exists(self):
-        return os.path.isdir(self.path)
+        return Path(self.path).is_dir()
 
     def _check_self(self):
         if not self.exists:
-            raise RTSLibNotInCFS("This %s does not exist in configFS"
-                                 % self.__class__.__name__)
+            raise RTSLibNotInCFSError(f"This {self.__class__.__name__} does not exist in configFS")
 
-    def _list_files(self, path, writable=None):
+    def _list_files(self, path, writable=None, readable=None):
         '''
         List files under a path depending on their owner's write permissions.
         @param path: The path under which the files are expected to be. If the
         path itself is not a directory, an empty list will be returned.
         @type path: str
-        @param writable: If None (default), returns all parameters, if True,
-        returns read-write parameters, if False, returns just the read-only
-        parameters.
+        @param writable: If None (default), return all files despite their
+        writability. If True, return only writable files. If False, return
+        only non-writable files.
         @type writable: bool or None
-        @return: List of file names filtered according to their write perms.
+        @param readable: If None (default), return all files despite their
+        readability. If True, return only readable files. If False, return
+        only non-readable files.
+        @type readable: bool or None
+        @return: List of file names filtered according to their
+        read/write perms.
         '''
-        if not os.path.isdir(path):
+        path = Path(path)
+        if not path.is_dir():
             return []
 
-        if writable is None:
-            names = os.listdir(path)
-        elif writable:
-            names = [name for name in os.listdir(path)
-                     if (os.stat("%s/%s" % (path, name))[stat.ST_MODE] \
-                         & stat.S_IWUSR)]
+        if writable is None and readable is None:
+            names = [p.name for p in path.glob('*') if p.is_file()]
         else:
-            names = [os.path.basename(name) for name in os.listdir(path)
-                     if not (os.stat("%s/%s" % (path, name))[stat.ST_MODE] \
-                             & stat.S_IWUSR)]
-        names.sort()
-        return names
+            names = []
+            for p in path.iterdir():
+                if p.is_file():
+                    sres = Path.stat(p)
+                    if (writable is not None and
+                            writable != ((sres[stat.ST_MODE] & stat.S_IWUSR) == stat.S_IWUSR)):
+                        continue
+                    if (readable is not None and
+                            readable != ((sres[stat.ST_MODE] & stat.S_IRUSR) == stat.S_IRUSR)):
+                        continue
+                    names.append(p.name)
+
+        return sorted(names)
 
     # CFSNode public stuff
 
-    def list_parameters(self, writable=None):
+    def list_parameters(self, writable=None, readable=None):
         '''
-        @param writable: If None (default), returns all parameters, if True,
-        returns read-write parameters, if False, returns just the read-only
-        parameters.
+        @param writable: If None (default), return all parameters despite
+        their writability. If True, return only writable parameters. If
+        False, return only non-writable parameters.
         @type writable: bool or None
+        @param readable: If None (default), return all parameters despite
+        their readability. If True, return only readable parameters. If
+        False, return only non-readable parameters.
+        @type readable: bool or None
         @return: The list of existing RFC-3720 parameter names.
         '''
         self._check_self()
-        path = "%s/param" % self.path
-        return self._list_files(path, writable)
+        path = f"{self.path}/param"
+        return self._list_files(path, writable, readable)
 
-    def list_attributes(self, writable=None):
+    def list_attributes(self, writable=None, readable=None):
         '''
-        @param writable: If None (default), returns all attributes, if True,
-        returns read-write attributes, if False, returns just the read-only
-        attributes.
+        @param writable: If None (default), return all files despite their
+        writability. If True, return only writable files. If False, return
+        only non-writable files.
         @type writable: bool or None
+        @param readable: If None (default), return all files despite their
+        readability. If True, return only readable files. If False, return
+        only non-readable files.
+        @type readable: bool or None
         @return: A list of existing attribute names as strings.
         '''
         self._check_self()
-        path = "%s/attrib" % self.path
-        return self._list_files(path, writable)
+        path = f"{self.path}/attrib"
+        return self._list_files(path, writable, readable)
 
     def set_attribute(self, attribute, value):
         '''
@@ -138,15 +162,14 @@ class CFSNode(object):
         @type value: string
         '''
         self._check_self()
-        path = "%s/attrib/%s" % (self.path, str(attribute))
-        if not os.path.isfile(path):
-            raise RTSLibError("Cannot find attribute: %s"
-                              % str(attribute))
+        path = Path(self.path) / 'attrib' / attribute
+        if not path.is_file():
+            raise RTSLibError(f"Cannot find attribute: {attribute!s}")
         else:
             try:
-                fwrite(path, "%s" % str(value))
+                fwrite(path, f"{value!s}")
             except Exception as e:
-                raise RTSLibError("Cannot set attribute %s: %s" % (attribute, e))
+                raise RTSLibError(f"Cannot set attribute {attribute}: {e}")
 
     def get_attribute(self, attribute):
         '''
@@ -154,9 +177,9 @@ class CFSNode(object):
         @return: The named attribute's value, as a string.
         '''
         self._check_self()
-        path = "%s/attrib/%s" % (self.path, str(attribute))
-        if not os.path.isfile(path):
-            raise RTSLibError("Cannot find attribute: %s" % attribute)
+        path = Path(self.path) / "attrib" / attribute
+        if not path.is_file():
+            raise RTSLibError(f"Cannot find attribute: {attribute}")
         else:
             return fread(path)
 
@@ -170,14 +193,14 @@ class CFSNode(object):
         @type value: string
         '''
         self._check_self()
-        path = "%s/param/%s" % (self.path, str(parameter))
-        if not os.path.isfile(path):
-            raise RTSLibError("Cannot find parameter: %s" % parameter)
+        path = Path(self.path) / "param" / parameter
+        if not path.is_file():
+            raise RTSLibError(f"Cannot find parameter: {parameter}")
         else:
             try:
-                fwrite(path, "%s\n" % str(value))
+                fwrite(path, f"{value!s}\n")
             except Exception as e:
-                raise RTSLibError("Cannot set parameter %s: %s" % (parameter, e))
+                raise RTSLibError(f"Cannot set parameter {parameter}: {e}")
 
     def get_parameter(self, parameter):
         '''
@@ -186,9 +209,9 @@ class CFSNode(object):
         @return: The named parameter value as a string.
         '''
         self._check_self()
-        path = "%s/param/%s" % (self.path, str(parameter))
-        if not os.path.isfile(path):
-            raise RTSLibError("Cannot find RFC-3720 parameter: %s" % parameter)
+        path = Path(self.path) / "param" / parameter
+        if not path.is_file():
+            raise RTSLibError(f"Cannot find RFC-3720 parameter: {parameter}")
         else:
             return fread(path)
 
@@ -199,28 +222,28 @@ class CFSNode(object):
         to delete it.
         '''
         if self.exists:
-            os.rmdir(self.path)
+            Path(self.path).rmdir()
 
     path = property(_get_path,
             doc="Get the configFS object path.")
-    exists = property(_exists,
-            doc="Is True as long as the underlying configFS object exists. " \
-                      + "If the underlying configFS objects gets deleted " \
-                      + "either by calling the delete() method, or by any " \
-                      + "other means, it will be False.")
+    exists = property(
+        _exists,
+        doc="Is True as long as the underlying configFS object exists. "
+            "If the underlying configFS objects gets deleted either by calling "
+            "the delete() method, or by any other means, it will be False.")
 
     def dump(self):
         d = {}
         attrs = {}
         params = {}
-        for item in self.list_attributes(writable=True):
+        for item in self.list_attributes(writable=True, readable=True):
             try:
                 attrs[item] = int(self.get_attribute(item))
             except ValueError:
                 attrs[item] = self.get_attribute(item)
         if attrs:
             d['attributes'] = attrs
-        for item in self.list_parameters(writable=True):
+        for item in self.list_parameters(writable=True, readable=True):
             params[item] = self.get_parameter(item)
         if params:
             d['parameters'] = params
